#property copyright "Copyright 2008, Ralph Ronnquist"

#include <pipschartdata.mqh>

extern bool alert = false;
extern string soundfile = "alert.wav";

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color4 Green

#define BEARCOLOR Red
#define BULLCOLOR Blue
#define SPIKECOLOR Silver

#define SPIKEWIDTH 2
#define BODYWIDTH 8

bool inited = false;

double first[];
double upper[];
double lower[];
double last[];

int init()
{
    IndicatorShortName( WindowExpertName() + ":" + pips );
    IndicatorDigits( Digits );
    SetIndexBuffer( 0, first );
    SetIndexStyle( 0, DRAW_NONE );
    SetIndexLabel( 0, "open" );

    SetIndexBuffer( 1, upper );
    SetIndexStyle( 1, DRAW_NONE );
    SetIndexLabel( 1, "high" );

    SetIndexBuffer( 2, lower );
    SetIndexStyle( 2, DRAW_NONE );
    SetIndexLabel( 2, "low" );

    SetIndexBuffer( 3, last );
    SetIndexStyle( 3, DRAW_LINE );
    SetIndexLabel( 3, "close" );

    PipsChartInit();
    inited = true;
    return(0);
}

int window_index = 1;

int deinit()
{
    for ( int i = ObjectsTotal() - 1; i >= 0; i-- ) {
        string name = ObjectName( i );
        if ( StringFind( name, window_index + "pc" ) == 0 )
            ObjectDelete( name );
    }
    return(0);
}

int start()
{
    if ( inited ) {
        window_index = WindowOnDropped();
        inited = false;
    } else {
        PipsChartUpdate( chart, TimeCurrent(), Close[ 0 ] );
    }
    for ( int i = pcBars( chart ) - 1; i >= 0; i-- ) {
        drawBar( i );
    }
    
    if ( alert ) {
        if ( pcCount( chart, 0 ) == 1 ) {
        
            Alert( "New PipsChart bar started" );
            if ( StringLen( soundfile ) != 0 )
                PlaySound( soundfile );
        }
    }
    return(0);
}

string wd[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };

void drawBar(int i)
{
    first[ i ] = EMPTY_VALUE;
    upper[ i ] = EMPTY_VALUE;
    lower[ i ] = EMPTY_VALUE;
    last[ i ] = EMPTY_VALUE;
    if ( pcOpen( chart, i ) == 0 ) {
        return;
    }
    first[ i ] = pcOpen( chart, i );
    upper[ i ] = pcHigh( chart, i );
    lower[ i ] = pcLow( chart, i );
    last[ i ] = pcClose( chart, i );
    if ( i+1 < pcBars( chart ) ) {
        datetime is = pcTime( chart, i );
        datetime was = pcTime( chart, i+1 );
        int is_day = TimeDayOfYear( is );
        int was_day = TimeDayOfYear( was );
        if ( is_day != was_day ) {
            int is_week = is_day / 7;
            int was_week = was_day / 7;
            string key = wd[ TimeDayOfWeek( is ) ] + " " + TimeToStr( is, TIME_DATE );
            string name = window_index + "pcday " + key;
            if ( is_week != was_week )
                drawVLine( name, Time[i], Gold, STYLE_SOLID );
            else
                drawVLine( name, Time[i], Silver, STYLE_DOT );
        }
    }
    drawBody( i, pcOpen( chart, i ), pcClose( chart, i ) );
    double hi = MathMax( pcOpen( chart, i ), pcClose( chart, i ) );
    double lo = MathMin( pcOpen( chart, i ), pcClose( chart, i ) );
    drawSpike( i, "hi", pcHigh( chart, i ), hi );
    drawSpike( i, "lo", lo, pcLow( chart, i ) );
}

void drawSpike(int i,string x,double hi,double lo)
{
    datetime t = Time[i];
    string name = window_index + "pcs" + i + x;
    drawLine( name, t, hi, lo, SPIKEWIDTH, SPIKECOLOR );
}

void drawBody(int i,double open,double close)
{
    datetime t = Time[i];
    string name = window_index + "pcb" + i ;
    if ( open > close )
        drawLine( name, t, open, close, BODYWIDTH, BEARCOLOR  );
    else
        drawLine( name, t, open, close, BODYWIDTH, BULLCOLOR );
    ObjectSetText( name, TimeToStr( pcTime( chart, i ) ) );
}

void drawLine(string name,datetime t,double hi,double lo,int w,color c)
{
    if ( ObjectFind( name ) < 0 ) {
        ObjectCreate( name, OBJ_TREND, window_index, t, hi, t, lo );
    } else {
        ObjectMove( name, 0, t, hi );
        ObjectMove( name, 1, t, lo );
    }
    ObjectSet( name, OBJPROP_RAY, false );
    ObjectSet( name, OBJPROP_BACK, true );
    ObjectSet( name, OBJPROP_WIDTH, w );
    ObjectSet( name, OBJPROP_COLOR, c );
}

void drawVLine(string name,datetime t,color c,int style)
{
    if ( ObjectFind( name ) < 0 ) {
        ObjectCreate( name, OBJ_VLINE, window_index, t, 0 );
    } else {
        ObjectMove( name, 0, t, 0 );
    }
    ObjectSet( name, OBJPROP_STYLE, style );
    ObjectSet( name, OBJPROP_BACK, true );
    //ObjectSet( name, OBJPROP_WIDTH, 2 );
    ObjectSet( name, OBJPROP_COLOR, c );
}

